package fmprotocol;


import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.Accessors;
import org.joda.time.DateTime;
import wei.yigulu.netty.AbstractDelimiterHandler;
import wei.yigulu.utils.DataConvertor;


/**
 * 未继承netty的数据帧处理拆包类
 *
 * @author 修唯xiuwei
 * @version 3.0
 */

public class FmRtuMasterDelimiterHandler extends AbstractDelimiterHandler {


    private static byte HEAD_TAIL_1 = (byte) 0xff;

    private static byte HEAD_TAIL_2 = (byte) 0x55;


    private static final int LENGTH = 56;

    /**
     * 判断是否是断包的最大时间间隔
     */
    @Setter
    @Getter
    @Accessors(chain = true)
    protected int maxTimeSpace = 250;


    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        log.info("-----------------------------------------------------------------");
        log.info("接收到原始的报文 ：" + DataConvertor.ByteBuf2String((ByteBuf) msg));
        if (isOverMaxLength((ByteBuf) msg)) {
            return;
        }

        if (!((cumulation.readableBytes() == 1 && cumulation.getByte(0) == HEAD_TAIL_1) ||
                cumulation.readableBytes() > 2 && checkHead(cumulation))) {
            log.warn("该段报文未以ff 55开头");
            int headIndex = getHeadIndex(cumulation.readerIndex(), cumulation.writerIndex(), cumulation, new byte[]{HEAD_TAIL_1, HEAD_TAIL_2});
            if (headIndex != -1) {
                log.warn("该段报文发现ff 55 的报文片段，,舍弃部分报文:" + DataConvertor.ByteBuf2String(cumulation.readBytes(headIndex+2)));
            } else {
                log.warn("该段报文未发现ff 55 的报文片段，整段报文全部清除");
                log.warn("这段字节中没有数据头,舍弃:" + DataConvertor.ByteBuf2String(cumulation.readBytes(cumulation.readableBytes())));
                clearCumulation();
                return;
            }
        }
        while (cumulation.readableBytes() >= LENGTH) {
            ByteBuf byteBuf = cumulation.readBytes(LENGTH);
            if (checkHead(byteBuf) && checkTail(byteBuf)) {
                log.info("发现符合ff 55开头，ff 55 结尾的56字节报文");
                ctx.fireChannelRead(byteBuf);
            }else{
                log.info("该56字节不符合ff 55开头，ff 55 结尾的格式");
                log.info( DataConvertor.ByteBuf2String(byteBuf));
                byteBuf.release();
            }
        }
        if (cumulation.readableBytes() == 0) {
            clearCumulation();
        } else {
            timeMark = DateTime.now();
        }

    }


    private boolean checkHead(ByteBuf msg) {
        if (msg.getByte(0) == HEAD_TAIL_1 & msg.getByte(1) == HEAD_TAIL_2) {
            return true;
        }
        return false;
    }


    private boolean checkTail(ByteBuf msg) {
        if (msg.getByte(msg.readableBytes() - 2) == HEAD_TAIL_1 & msg.getByte(msg.readableBytes() - 1) == HEAD_TAIL_2) {
            return true;
        }
        return false;
    }
}
