package fmprotocol;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.EventLoop;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import wei.yigulu.utils.DataConvertor;

import java.util.concurrent.TimeUnit;

/**
 * 16进制报文解析工具
 *
 * @author xiuwei
 */
@Data
@Slf4j
public class FmRtuMasterHandler extends SimpleChannelInboundHandler<ByteBuf> {


    private static final int LEN = 52;

    private FmRtuMasterBuilder FmMaster;

    public FmRtuMasterHandler(FmRtuMasterBuilder FmMaster) {
        this.FmMaster = FmMaster;
    }

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        FmMaster.getLog().info("-----连接串口{}成功-----", this.FmMaster.getCommPortId());
    }

    /**
     * channel断连及不稳定时调用的方法
     *
     * @param ctx
     * @throws Exception
     */
    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        FmMaster.getLog().error("串口{}连接中断,正在启动重连机制... ", this.FmMaster.getCommPortId());
        //在客户端与服务端连接过程中如果断连，就会调用的方法
        final EventLoop eventLoop = ctx.channel().eventLoop();
        eventLoop.schedule(() -> {
            FmMaster.getLog().info("正在重连串口{}", this.FmMaster.getCommPortId());
            this.FmMaster.create();
        }, 3L, TimeUnit.SECONDS);
    }

    /**
     * channel连接及传输报错时调用的方法
     *
     * @param ctx
     * @param cause
     */
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        FmMaster.getLog().error("串口异常消息:", cause);
    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, ByteBuf msg) throws Exception {
       FmMaster.getLog().info("接收到串口{}发来数据帧: <= " + DataConvertor.ByteBuf2String(msg), this.FmMaster.getCommPortId());
        if (msg.readableBytes() == LEN) {
            DataBeanPacker.getInstance().decodeAnRecode(msg);
        }
    }
}
